# Phantom script by NowakPL <nowak@xpam.de>

local config = {
	start_time = 8,
	phantom_economy = {
		income_bonus = {
			ally = { mass = 0.3, energy = 0.3 },
			enemy = { mass = 0.5, energy = 0.5 },
			kill = { mass = 0.2, energy = 0.2 },
		},
		min_stored_ratio = { mass = 0.5, energy = 0.5 },
		max_stored_ratio = { mass = 0.8, energy = 0.8 },
	},
}

#
# PHANTOM CODE BEGINS HERE
#

local cur = {
	phantom = nil,
	trash = nil,
	last_give_resource = { mass = 0, energy = 0 },
}

local ParentBeginSession = BeginSession
function BeginSession()
	ParentBeginSession()

	cur.trash = TrashBag()
	ForkThread(PhantomMainThread)
end

function PhantomMainThread()
	for army, brain in ArmyBrains do
		brain.CalculateScore = PhantomCalculateScore
	end

	WaitSeconds(config.start_time * 60)

	local players = {}
	for army, brain in ArmyBrains do
		if ArmyIsCivilian(army) == false and ArmyIsOutOfGame(army) == false then
			table.insert(players, brain)
		end
	end

	cur.phantom = players[Random(table.getn(players))]
	cur.phantom_army = cur.phantom:GetArmyIndex()
	cur.trash:Add(ForkThread(PhantomNotifyThread))
	cur.trash:Add(ForkThread(PhantomResourceThread))
	
	local end_game = false
	local old_players_alive = -1
	while end_game == false do
		WaitSeconds(0.1)

		local players_alive = 0
		local phantom_alive = 0
		for army, brain in ArmyBrains do
			if ArmyIsCivilian(army) == false then
				SetAlliedVictory(army, false)
				if ArmyIsOutOfGame(army) == false then
					if army == cur.phantom_army then
						phantom_alive = 1
					else
						players_alive = players_alive + 1
					end
				end
			end
		end
		if old_players_alive >= 0 and old_players_alive != players_alive then
			PrintText('Innocent player has been killed', 20, 'ffff2020', 10, 'center')
		end
		if phantom_alive >= 1 and players_alive < 1 then
			PrintText('Phantom wins', 20, nil, 10, 'center')
			WaitSeconds(5)
			end_game = true
		end
		if phantom_alive < 1 and players_alive >= 1 then
			PrintText('Phantom fails', 20, nil, 10, 'center')
			WaitSeconds(5)
			end_game = true
		end
		if phantom_alive < 1 and players_alive < 1 then
			PrintText('Everyone fails :)', 20, 'ffff2020', 10, 'center')
			WaitSeconds(5)
			end_game = true
		end
		old_players_alive = players_alive
	end

	for army, brain in ArmyBrains do
		if ArmyIsCivilian(army) == false and ArmyIsOutOfGame(army) == false then
			brain:OnVictory()
		end
	end

	cur.trash:Destroy()
end

function PhantomCalculateScore(self)
	local massValueDestroyed = self:GetArmyStat("Enemies_MassValue_Destroyed",0.0).Value
	local massValueLost = self:GetArmyStat("Units_MassValue_Lost",0.0).Value
	local energyValueDestroyed = self:GetArmyStat("Enemies_EnergyValue_Destroyed",0.0).Value
	local energyValueLost = self:GetArmyStat("Units_EnergyValue_Lost",0.0).Value

	local energyValueCoefficient = 0.02
	return math.floor((massValueDestroyed - massValueLost) + (energyValueDestroyed - energyValueLost) * energyValueCoefficient)
end

function PhantomNotifyThread()
	if cur.phantom_army == GetFocusArmy() then
		PrintText('Eliminate other players', 20, 'ffff2020', 15, 'center')
	else
		PrintText('Eliminate the Phantom', 20, 'ffffff00', 15, 'center')
	end
end

function PhantomResourceThread()
	while true do
		WaitSeconds(0.1)

		local mass = 0
		local energy = 0
		local mass_bonus = 0
		local energy_bonus = 0
		for army, brain in ArmyBrains do
			if ArmyIsCivilian(army) == false and army != cur.phantom_army then
				if ArmyIsOutOfGame(army) == false then
					if IsAlly(army, cur.phantom_army) then
						mass = mass + config.phantom_economy.income_bonus.ally.mass * brain:GetEconomyIncome('MASS')
						energy = energy + config.phantom_economy.income_bonus.ally.energy * brain:GetEconomyIncome('ENERGY')
					else
						mass = mass + config.phantom_economy.income_bonus.enemy.mass * brain:GetEconomyIncome('MASS')
						energy = energy + config.phantom_economy.income_bonus.enemy.energy * brain:GetEconomyIncome('ENERGY')
					end
				else
					mass_bonus = mass_bonus + config.phantom_economy.income_bonus.kill.mass
					energy_bonus = energy_bonus + config.phantom_economy.income_bonus.kill.energy
				end
			end
		end
		local give_resource = { mass = 0, energy = 0 }
		if cur.phantom:GetEconomyStoredRatio('MASS') < config.phantom_economy.max_stored_ratio.mass then
			local income = math.max(0, math.floor(0.5 + (cur.phantom:GetEconomyIncome('MASS') - cur.last_give_resource.mass) * 100) / 100)
			local give = mass + income * mass_bonus
			local f = 1 - math.min(1, math.max(0, (cur.phantom:GetEconomyStoredRatio('MASS') - config.phantom_economy.min_stored_ratio.mass) / (config.phantom_economy.max_stored_ratio.mass - config.phantom_economy.min_stored_ratio.mass)))
			give_resource.mass = f * give
		end
		if cur.phantom:GetEconomyStoredRatio('ENERGY') < config.phantom_economy.max_stored_ratio.energy then
			local income = math.max(0, math.floor(0.5 + (cur.phantom:GetEconomyIncome('ENERGY') - cur.last_give_resource.energy) * 100) / 100)
			local give = energy + income * energy_bonus
			local f = 1 - math.min(1, math.max(0, (cur.phantom:GetEconomyStoredRatio('ENERGY') - config.phantom_economy.min_stored_ratio.energy) / (config.phantom_economy.max_stored_ratio.energy - config.phantom_economy.min_stored_ratio.energy)))
			give_resource.energy = f * give
		end
		if give_resource.mass then
			cur.phantom:GiveResource('MASS', give_resource.mass)
		end
		if give_resource.energy then
			cur.phantom:GiveResource('ENERGY', give_resource.energy)
		end
		cur.last_give_resource.mass = give_resource.mass
		cur.last_give_resource.energy = give_resource.energy
	end

end
